import { useState, useEffect } from 'react';
import AddIcon from '@mui/icons-material/Add';
import RemoveIcon from '@mui/icons-material/Remove';

import styles from './ProductQuantity.module.scss';

export interface Event {
    name: string;
    quantity: number;
}

export interface ProductQuantityProps {
    inputQuantity?: number;
    minQuantity?: number;
    maxQuantity?: number;
    editable?: boolean;
    preventDefault?: boolean;
    add?: (event: Event) => void;
    remove?: (event: Event) => void;
    change?: (event: Event) => void;
    focus?: (event: Event) => void;
    blur?: (event: Event) => void;
}

function ProductQuantity(props: ProductQuantityProps) {
    const {
        inputQuantity = 0,
        minQuantity = 0,
        maxQuantity,
        editable,
        preventDefault,
        add,
        remove,
        change,
        focus,
        blur,
    } = props;
    const initialQuantity = inputQuantity || minQuantity;
    const [quantity, setQuantity] = useState(initialQuantity);

    useEffect(() => {
        setQuantity(() => initialQuantity);
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [inputQuantity, minQuantity]);

    const handleAdd = () => {
        const newQuantity = quantity + 1;
        if (
            (maxQuantity && (maxQuantity > newQuantity)) ||
            (maxQuantity === undefined || maxQuantity === null)
        ) {
            if (!preventDefault) {
                setQuantity(newQuantity);
                if (add instanceof Function) {
                    add({
                        name: 'add',
                        quantity: newQuantity
                    });
                }
            } else {
                if (remove instanceof Function) {
                    remove({
                        name: 'remove',
                        quantity,
                    });
                }
            }
        }
    }

    const handleRemove = () => {
        if (
            quantity > 0 &&
            (
                quantity > minQuantity ||
                minQuantity === undefined ||
                minQuantity === null
            )
        ) {
            if (!preventDefault) {
                const newQuantity = quantity - 1;
                setQuantity(newQuantity);
                if (remove instanceof Function) {
                    remove({
                        name: 'remove',
                        quantity: newQuantity,
                    });
                }
            } else {
                if (remove instanceof Function) {
                    remove({
                        name: 'remove',
                        quantity,
                    });
                }
            }
        }
    }

    const handleChange = () => {
        if (change instanceof Function) {
            change({
                name: 'change',
                quantity
            });
        }
    }

    const handleBlur = () => {
        if (blur instanceof Function) {
            blur({
                name: 'blur',
                quantity
            });
        }
    }

    const handleFocus = () => {
        if (focus instanceof Function) {
            focus({
                name: 'focus',
                quantity
            });
        }
    }

    return (
        <div className={styles.stepper}>
            <button
                type="button"
                className={styles.remove}
                onClick={handleRemove}
                disabled={editable === false}>
                <RemoveIcon />
            </button>
            <input
                type="text"
                className={styles.number}
                onChange={handleChange}
                onBlur={handleBlur}
                onFocus={handleFocus}
                onKeyPress={(event) => {
                    if (!/[0-9]/.test(event.key)) {
                        event.preventDefault();
                    }
                }}
                value={quantity}
                disabled={editable === false}
                required
            />
            <button
                type="button"
                className={styles.add}
                onClick={handleAdd}
                disabled={editable === false}>
                <AddIcon />
            </button>
        </div>
    )
}

export default ProductQuantity;